from swidget.swidgetdevice import (
    DeviceType,
    SwidgetDevice
)


class SwidgetOutlet(SwidgetDevice):

    def __init__(self, host,  token_name: str, secret_key: str, ssl: bool, use_websockets: bool) -> None:
        super().__init__(host=host, token_name=token_name, secret_key=secret_key, ssl=ssl, use_websockets=use_websockets)
        self._device_type = DeviceType.Outlet


from swidget.swidgetdevice import (
    DeviceType,
)
from swidget.swidgetswitch import SwidgetSwitch


class SwidgetTimerSwitch(SwidgetSwitch):

    def __init__(self, host,  token_name: str, secret_key: str, ssl: bool, use_websockets: bool) -> None:
        super().__init__(host=host, token_name=token_name, secret_key=secret_key, ssl=ssl, use_websockets=use_websockets)
        self._device_type = DeviceType.TimerSwitch

    async def set_countdown_timer(self, minutes):
        """Set the countdown timer."""
        await self.send_command(
            assembly="host", component="0", function="timer", command={"duration": minutes}
        )


"""Python interface for Swidget smart home devices.

All common, shared functionalities are available through `SwidgetDevice` class::

    x = SwidgetDevice("192.168.1.1", "mypassword")
    print(x.sys_info)

For device type specific actions `SwidgetDimmer`, `SwidgetOutlet`, or `SwidgetSwitch`
 should be used instead.

Module-specific errors are raised as `SwidgetException` and are expected
to be handled by the user of the library.
"""
from importlib_metadata import version  # type: ignore

from swidget.discovery import discover_devices, discover_single, SwidgetDiscoveredDevice
from swidget.exceptions import SwidgetException
from swidget.swidgetdevice import DeviceType, SwidgetAssembly, SwidgetDevice, SwidgetComponent
from swidget.swidgetdimmer import SwidgetDimmer
from swidget.swidgetoutlet import SwidgetOutlet
from swidget.swidgetswitch import SwidgetSwitch
from swidget.swidgettimerswitch import SwidgetTimerSwitch


__version__ = version("python-swidget")


__all__ = [
    "discover_devices",
    "discover_single",
    "SwidgetDiscoveredDevice",
    "SwidgetException",
    "DeviceType",
    "SwidgetAssembly",
    "SwidgetDevice",
    "SwidgetComponent",
    "SwidgetDimmer",
    "SwidgetOutlet",
    "SwidgetSwitch",
    "SwidgetTimerSwitch",
]


import asyncio
import json
import logging
import socket
from typing import Awaitable, Callable, Dict, Optional, Type, cast
from urllib.parse import urlparse

import ssdp

from swidget.swidgetdevice import DeviceType, SwidgetDevice
from .swidgetdimmer import SwidgetDimmer
from .swidgetoutlet import SwidgetOutlet
from .swidgetswitch import SwidgetSwitch
from .swidgettimerswitch import SwidgetTimerSwitch
from .exceptions import SwidgetException

RESPONSE_SEC = 5
SWIDGET_ST = "urn:swidget:pico:1"
_LOGGER = logging.getLogger(__name__)
devices = dict()


class SwidgetDiscoveredDevice:
    def __init__(self, mac: str, host: str, friendly_name: str = "Swidget Discovered Device"):
        self.mac = mac
        self.host = host
        self.friendly_name = friendly_name


class SwidgetProtocol(ssdp.SimpleServiceDiscoveryProtocol):
    """Protocol to handle responses and requests."""
    def response_received(self, response: ssdp.SSDPResponse, addr: tuple):
        "Handle an incoming response."
        headers = {h[0]: h[1] for h in response.headers}
        mac_address = headers["USN"].split("-")[-1]
        ip_address = urlparse(headers["LOCATION"]).hostname
        if headers["ST"] == SWIDGET_ST:
            device_type = headers["SERVER"].split(" ")[1].split("+")[0]
            insert_type = headers["SERVER"].split(" ")[1].split("+")[1].split("/")[0]
            friendly_name = f"Swidget {device_type} w/{insert_type} insert"
            devices[mac_address] = SwidgetDiscoveredDevice(mac_address, ip_address, friendly_name)


async def discover_devices(timeout=RESPONSE_SEC):
    loop = asyncio.get_event_loop()
    transport, protocol = await loop.create_datagram_endpoint(
        SwidgetProtocol, family=socket.AF_INET
    )

    # Send out an M-SEARCH request, requesting Swidget service types.
    search_request = ssdp.SSDPRequest(
        "M-SEARCH",
        headers={
            "HOST": "239.255.255.250:1900",
            "MAN": '"ssdp:discover"',
            "MX": timeout,
            "ST": SWIDGET_ST,
        },
    )
    search_request.sendto(transport, (SwidgetProtocol.MULTICAST_ADDRESS, 1900))
    await asyncio.sleep(timeout)
    _LOGGER.debug(f"Found the following Swidget devices from SSDP discovery: {devices}")
    return devices


async def discover_single(host: str, token_name: str, password: str, ssl: bool, use_websockets: bool) -> SwidgetDevice:
    """Discover a single device by the given IP address.

    :param host: Hostname of device to query
    :rtype: SwidgetDevice
    :return: Object for querying/controlling found device.
    """
    swidget_device = SwidgetDevice(host, token_name, password, ssl, use_websockets)
    await swidget_device.get_summary()
    device_type = swidget_device.device_type
    device_class = _get_device_class(device_type)
    dev = device_class(host, token_name, password, False, use_websockets)
    await dev.update()
    return dev


def _get_device_class(device_type: str) -> Type[SwidgetDevice]:
    """Find SmartDevice subclass for device described by passed data."""
    if device_type == "outlet":
        return SwidgetOutlet
    elif device_type == "switch":
        return SwidgetSwitch
    elif device_type == "dimmer":
        return SwidgetDimmer
    elif device_type == "pana_switch": # This is the timer switch
        return SwidgetTimerSwitch
    elif device_type == "relay_switch":
        return SwidgetSwitch
    raise SwidgetException("Unknown device type: %s" % device_type)



"""python-swidget cli tool."""
import asyncio
import logging
import sys
from pprint import pformat as pf
from typing import cast

import asyncclick as click

from swidget import (
    discover_devices,
    discover_single,
    SwidgetDevice,
    SwidgetDimmer,
    SwidgetSwitch,
    SwidgetOutlet,
    SwidgetTimerSwitch
)

TYPE_TO_CLASS = {
    "dimmer": SwidgetDimmer,
    "switch": SwidgetSwitch,
    "outlet": SwidgetOutlet,
    "pana_switch": SwidgetTimerSwitch
}

click.anyio_backend = "asyncio"


pass_dev = click.make_pass_decorator(SwidgetDevice)


@click.group(invoke_without_command=True)
@click.option(
    "--host",
    envvar="SWIDGET_HOST",
    required=False,
    help="The host name or IP address of the device to connect to.",
)
@click.option(
    "--password",
    envvar="SWIDGET_PASSWORD",
    required=False,
    help="The password of the device to connect to.",
)
@click.option("-d", "--debug",
    envvar="SWIDGET_DEBUG",
    default=False,
    is_flag=True)
@click.option(
    "--type",
    envvar="SWIDGET_TYPE",
    default=None,
    type=click.Choice(list(TYPE_TO_CLASS), case_sensitive=False),
)
@click.version_option(package_name="python-swidget")
@click.pass_context
async def cli(ctx, host, password, debug, type):
    """A tool for controlling Swidget smart home devices."""  # noqa
    # no need to perform any checks if we are just displaying the help
    if sys.argv[-1] == "--help":
        # Context object is required to avoid crashing on sub-groups
        ctx.obj = SwidgetDevice(None)
        return

    if debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    if ctx.invoked_subcommand == "discover":
        return

    if host is None:
        click.echo("No host name given, trying discovery..")
        await ctx.invoke(discover)
        return

    if type is not None:
        dev = TYPE_TO_CLASS[type](host, password, False, False)
    else:
        click.echo("No --type defined, discovering..")
        dev = await discover_single(host, password, False, False)

    await dev.update()
    ctx.obj = dev

    if ctx.invoked_subcommand is None:
        await ctx.invoke(state)


@cli.group()
@pass_dev
def wifi(dev):
    """Commands to control wifi settings."""

@wifi.command()
@click.argument("ssid")
@click.option("--password", prompt=True, hide_input=True)
@click.option("--keytype", default=3)
@pass_dev
async def join(dev: SwidgetDevice, ssid, password, keytype):
    """Join the given wifi network."""
    click.echo(f"Asking the device to connect to {ssid}..")
    res = await dev.wifi_join(ssid, password, keytype=keytype)
    click.echo(
        f"Response: {res} - if the device is not able to join the network, it will revert back to its previous state."
    )

    return res


@cli.command()
@click.option("--timeout", default=5, required=False)
@click.pass_context
async def discover(ctx, timeout):
    """Discover devices in the network."""
    click.echo(f"Discovering devices via SSDP...waiting up to {timeout} seconds")

    devices = await discover_devices(timeout)
    for device in devices.values():
        click.echo(f"{device.host}[{device.mac}] - {device.friendly_name}")

@cli.command()
@pass_dev
async def hwinfo(dev):
    """Print out full system information."""
    click.echo(click.style("== HW info ==", bold=True))
    click.echo(pf(dev.hw_info))
    return dev.hw_info


@cli.command()
@pass_dev
async def state(dev: SwidgetDevice):
    """Print out device state and versions."""
    click.echo(click.style(f"== {dev.friendly_name} - {dev.model} ==", bold=True))
    click.echo(f"\tHost: {dev.ip_address}")
    click.echo(
        click.style(
            "\tDevice state: {}\n".format("ON" if dev.is_on else "OFF"),
            fg="green" if dev.is_on else "red",
        )
    )

    click.echo(click.style("\t== Generic information ==", bold=True))
    click.echo(f"\tHardware:     {dev.hw_info['model']}")
    click.echo(f"\tSoftware:     {dev.hw_info['version']}")
    click.echo(f"\tMAC (rssi):   {dev.mac_address} ({dev.rssi})")

    click.echo(click.style("\n\t== Current State ==", bold=True))
    for info_name, info_data in dev.realtime_values.items():
        if isinstance(info_data, list):
            click.echo(f"\t{info_name}:")
            for item in info_data:
                click.echo(f"\t\t{item}")
        else:
            click.echo(f"\t{info_name}: {info_data}")

    click.echo(click.style("\n\t== Host Features ==", bold=True))
    for function in dev.host_features:
        click.echo(click.style(f"\t+ {function}", fg="green"))


    click.echo(click.style("\n\t== Insert Features ==", bold=True))
    for function in dev.insert_features:
        click.echo(click.style(f"\t+ {function}", fg="green"))



@cli.command()
@pass_dev
@click.argument("assembly")
@click.argument("component")
@click.argument("function")
@click.argument("command")
async def raw_command(dev: SwidgetDevice, assembly, component, function, command):
    """Run a raw command on the device."""
    import ast

    if parameters is not None:
        parameters = ast.literal_eval(parameters)

    res = await dev.send_command(assembly, component, function, command)

    click.echo(res)
    return res



@cli.command()
@click.argument("brightness", type=click.IntRange(0, 100), default=None, required=False)
@pass_dev
async def brightness(dev: SwidgetDimmer, brightness: int):
    """Get or set brightness."""
    if not dev.is_dimmer:
        click.echo("This device does not support brightness.")
        return

    if brightness is None:
        click.echo(f"Brightness: {dev.brightness}")
    else:
        click.echo(f"Setting brightness to {brightness}")
        return await dev.set_brightness(brightness)


@cli.command()
@pass_dev
async def blink(dev):
    """Set the device insert to blink"""
    click.echo(f"Requesting the device to blink")
    return await dev.blink()


@cli.command()
@pass_dev
async def on(dev: SwidgetDevice):
    """Turn the device on."""
    click.echo(f"Turning on {dev.friendly_name}")
    return await dev.turn_on()


@cli.command()
@pass_dev
async def off(dev: SwidgetDevice):
    """Turn the device off."""
    click.echo(f"Turning off {dev.friendly_name}")
    return await dev.turn_off()


if __name__ == "__main__":
    cli()



import asyncio
from datetime import datetime
import logging
import json

import aiohttp

_LOGGER = logging.getLogger(__name__)

ERROR_AUTH_FAILURE = "Authorization failure"
ERROR_TOO_MANY_RETRIES = "Too many retries"
ERROR_UNKNOWN = "Unknown"

MAX_FAILED_ATTEMPTS = 5

STATE_CONNECTED = "connected"
STATE_DISCONNECTED = "disconnected"
STATE_STARTING = "starting"
STATE_STOPPED = "stopped"


class SwidgetWebsocket:
    """A websocket connection to a Swidget Device"""

    # pylint: disable=too-many-instance-attributes

    def __init__(
        self,
        host,
        token_name,
        secret_key,
        callback,
        session=None,
        verify_ssl=False,
    ):

        self.session = session or aiohttp.ClientSession()
        self.uri = self._get_uri(host, token_name, secret_key)
        self.callback = callback
        self._ssl = False if verify_ssl is False else None
        self._state = None
        self.failed_attempts = 0
        self._error_reason = None

    @property
    def state(self):
        """Return the current state."""
        return self._state

    @state.setter
    def state(self, value):
        """Set the state."""
        self._state = value

    @staticmethod
    def _get_uri(host, token_name, secret_key):
        """Generate the websocket URI"""
        return f"wss://{host}/api/v1/sock?{token_name}={secret_key}"

    async def running(self):
        """Open a persistent websocket connection and act on events."""
        self.state = STATE_STARTING

        try:
            headers = {'Connection': 'Upgrade'}
            async with self.session.ws_connect(self.uri, headers=headers, verify_ssl=False) as self.ws_client:
                _LOGGER.error("AIOHTTP websocket connected")
                self.state = STATE_CONNECTED
                self.failed_attempts = 0
                self.send_str(json.dumps({"type": "summary", "request_id": "1"}))
                self.send_str(json.dumps({"type": "state", "request_id": "2"}))
                async for message in self.ws_client:
                    _LOGGER.error(f'AIOHTTP websocket message {message.type}')
                    if self.state == STATE_STOPPED:
                        break

                    if message.type == aiohttp.WSMsgType.TEXT:
                        msg = message.json()
                        await self.callback(msg)

                    elif message.type == aiohttp.WSMsgType.CLOSED:
                        _LOGGER.warning("AIOHTTP websocket connection closed")
                        break

                    elif message.type == aiohttp.WSMsgType.ERROR:
                        _LOGGER.error(f"AIOHTTP websocket error. Message-type: {message.type} {message}")
                        break

        except aiohttp.ClientResponseError as error:
            _LOGGER.error(f'AIOHTTP ClientResponseError')
            if error.code == 401:
                _LOGGER.error(f"Credentials rejected: {error}")
                self._error_reason = ERROR_AUTH_FAILURE
            else:
                _LOGGER.error(f"Unexpected response received: {error}")
                self._error_reason = ERROR_UNKNOWN
            self.state = STATE_STOPPED
        except (aiohttp.ClientConnectionError, asyncio.TimeoutError) as error:
            _LOGGER.error(f'AIOHTTP ClientConnectionError, asyncio.TimeoutError')
            if self.failed_attempts >= MAX_FAILED_ATTEMPTS:
                self._error_reason = ERROR_TOO_MANY_RETRIES
                self.state = STATE_STOPPED
            elif self.state != STATE_STOPPED:
                retry_delay = min(2 ** (self.failed_attempts - 1) * 30, 300)
                self.failed_attempts += 1
                _LOGGER.exception(f"Websocket connection failed, retrying in {retry_delay}s: {error}")
                self.state = STATE_DISCONNECTED
                await asyncio.sleep(retry_delay)
        except Exception as error:  # pylint: disable=broad-except
        _LOGGER.error(f'AIOHTTP Exception')
            if self.state != STATE_STOPPED:
                _LOGGER.exception(f"Unexpected exception occurred: {error}")
                self._error_reason = ERROR_UNKNOWN
                self.state = STATE_STOPPED
        else:
            if self.state != STATE_STOPPED:
                self.state = STATE_DISCONNECTED

                await asyncio.sleep(5)

    async def send_str(self, message):
        # _LOGGER.error(f"Sending Message: {message}")
        message = str(message)
        await self.ws_client.send_str(f'{message}')

    async def listen(self):
        """Close the listening websocket."""
        self.failed_attempts = 0
        while self.state != STATE_STOPPED:
            await self.running()

    def close(self):
        """Close the listening websocket."""
        self.state = STATE_STOPPED
        self.ws_client.close()



import json
import logging
import time

from aiohttp import ClientSession, TCPConnector
from enum import Enum
from typing import Dict, List, Set

from .exceptions import SwidgetException
from .websocket import SwidgetWebsocket

_LOGGER = logging.getLogger(__name__)


class DeviceType(Enum):
    """Device type enum."""

    Dimmer = "dimmer"
    Outlet = "outlet"
    Switch = "switch"
    TimerSwitch = "pana_switch"
    RelaySwitch = "relay_switch"
    Unknown = -1


class SwidgetDevice:
    def __init__(self, host, token_name, secret_key, ssl=False, use_websockets=True):
        self.token_name = token_name
        self.ip_address = host
        self.ssl = ssl
        self.secret_key = secret_key
        self.use_websockets = use_websockets
        self.device_type = DeviceType.Unknown
        headers = {self.token_name: self.secret_key}
        connector = TCPConnector(force_close=True)
        self._session = ClientSession(headers=headers, connector=connector)
        self._last_update = None
        if self.use_websockets:
            self._websocket = SwidgetWebsocket(
                host=self.ip_address,
                token_name=self.token_name,
                secret_key=self.secret_key,
                callback=self.message_callback,
                session=self._session)

    def get_websocket(self):
        return self._websocket
        
    def set_countdown_timer(self, minutes):
        raise NotImplementedError()

    def stop(self):
        """Stop the websocket."""
        if self._websocket is not None:
            self._websocket.stop()

    async def message_callback(self, message):
        """Entrypoint for a websocket callback"""
        if message["request_id"] == "summary":
            await self.process_summary(message)
        elif message["request_id"] == "state" or message["request_id"] == "DYNAMIC_UPDATE" or message["request_id"] == "command":
            await self.process_state(message)

    async def get_summary(self):
        """Get a summary of the device over HTTP"""
        async with self._session.get(
            url=f"https://{self.ip_address}/api/v1/summary", ssl=self.ssl
        ) as response:
            summary = await response.json()
        await self.process_summary(summary)

    async def process_summary(self, summary):
        """ Process the data around the summary of the device"""
        self.model = summary["model"]
        self.mac_address = summary["mac"]
        self.version = summary["version"]
        self.assemblies = {
            "host": SwidgetAssembly(summary["host"]),
            "insert": SwidgetAssembly(summary["insert"]),
        }
        self.device_type = self.assemblies['host'].type
        self.insert_type = self.assemblies['insert'].type
        self.id = self.assemblies['host'].id
        self._last_update = int(time.time())

    async def get_state(self):
        """ Get the state of the device over HTTP"""
        async with self._session.get(
            url=f"https://{self.ip_address}/api/v1/state", ssl=self.ssl
        ) as response:
            state = await response.json()
        await self.process_state(state)

    async def process_state(self, state):
        """ Process any information about the state of the device or insert"""
        # State is not always in the state (during callback)
        try:
            self.rssi = state["connection"]["rssi"]
        except:
            pass

        for assembly in self.assemblies:
            for id, component in self.assemblies[assembly].components.items():
                try:
                    component.functions.update(state[assembly]["components"][id])
                except:
                    pass
        self._last_update = int(time.time())

    async def update(self):
        if self._last_update is None:
            _LOGGER.debug("Performing the initial update to obtain sysinfo")
        await self.get_summary()
        await self.get_state()

    async def send_config(self, payload: dict):
        data = json.dumps({"type":"config","request_id":"abcd", "payload": payload})
        await self._websocket.send_str(data)

    async def send_command(
        self, assembly: str, component: str, function: str, command: dict
    ):
        """Send a command to the Swidget device either using a HTTP call or the existing websocket"""
        data = {assembly: {"components": {component: {function: command}}}}

        if self.use_websockets:
            data = json.dumps({"type": "command",
                               "request_id": "command",
                               "payload": data
                               })
            await self._websocket.send_str(data)
        else:
            async with self._session.post(
                url=f"https://{self.ip_address}/api/v1/command",
                ssl=self.ssl,
                data=json.dumps(data),
            ) as response:
                state = await response.json()

            function_value = state[assembly]["components"][component][function]
            self.assemblies[assembly].components[component].functions[function] = function_value  # fmt: skip

    async def ping(self):
        """Ping the device to ensure it's devices

        :raises SwidgetException: Raise the exception if there we are unable to connect to the Swidget device
        """
        try:
            async with self._session.get(
                url=f"https://{self.ip_address}/ping",
                ssl=self.ssl
            ) as response:
                return response.text
        except:
            raise SwidgetException

    async def blink(self):
        """Make the device LED blink

        :raises SwidgetException: Raise the exception if there we are unable to connect to the Swidget device
        """
        try:
            async with self._session.get(
                url=f"https://{self.ip_address}/blink?x-user-key=dqMMBX9deuwtkkp784ewTjqo76IYfThV",
                ssl=self.ssl
            ) as response:
                return response.text
        except:
            raise SwidgetException

    @property
    def hw_info(self) -> Dict:
        """
        Return hardware information.

        This returns just a selection of attributes that are related to hardware.
        """
        return {
            "version": self.version,
            "mac_address": self.mac_address,
            "type": self.device_type,
            "id": self.id,
            "model": self.model,
            "insert_type": self.insert_type,
            "insert_features": self.insert_features,
            "host_features": self.host_features,
            "rssi": self.rssi
        }

    def get_child_consumption(self, plug_id=0):
        """Get the power consumption of a plug in watts."""
        if plug_id == "all":
            return_dict = {}
            for id, properties in self.assemblies['host'].components.items():
                try:
                    return_dict[f"power_{id}"] = properties.functions['power']['current']
                except KeyError: # Hits this when there is no power metering
                    return None
            return return_dict
        return self.assemblies['host'].components[str(plug_id)].functions['power']['current']

    async def total_consumption(self):
        """Get the total power consumption in watts."""
        total_consumption = 0
        for id, properties in self.assemblies['host'].components.items():
            total_consumption += properties.functions['power']['current']
        return total_consumption

    @property
    def realtime_values(self):
        """Get a dict of realtime value attributes from the insert and host

        :return: A dictionary of insert sensor values and power consumption values
        :rtype: dict
        """
        return_dict = {}
        for feature in self.insert_features:
            return_dict.update(self.get_function_values(feature))
        return_dict.update({'rssi': self.rssi})
        power_values = self.get_child_consumption("all")
        if power_values:
            return_dict.update(power_values)
        return return_dict

    @property
    def host_features(self) -> List[str]:
        """Return a set of features that the host supports."""
        try:
            return list(self.assemblies['host'].components['0'].functions.keys())
        except KeyError:
            _LOGGER.debug("Host does not have feature information")
            return set()

    @property
    def insert_features(self) -> List[str]:
        """Return a set of features that the insert supports."""
        try:
            return list(self.assemblies['insert'].components.keys())
        except KeyError:
            _LOGGER.debug("Insert does not have feature information")
            return set()

    def get_function_values(self, function: str):
        """Return the values of an insert function."""
        return_values = dict()
        for function, data in self.assemblies['insert'].components[function].functions.items():
            if function == "occupied":
                return_values[function] = data['state']
            elif function == "toggle":
                pass
            else:
                return_values[function] = data['now']
        return return_values

    def get_sensor_value(self, function, sensor):
        """Return the value of a sensor given a function and sensor"""
        if sensor == "occupied":
            return self.assemblies['insert'].components[function].functions['occupied']['state']
        else:
            return self.assemblies['insert'].components[function].functions[sensor]['now']

    @property
    def is_outlet(self) -> bool:
        """Return True if the device is an outlet."""
        return self.device_type == "outlet"

    @property
    def is_switch(self) -> bool:
        """Return True if the device is a switch"""
        return self.device_type == "switch" or self.device_type == "pana_switch" or self.device_type == "relay_switch"

    @property
    def is_pana_switch(self) -> bool:
        """Return True if the device is a pana_switch"""
        return self.device_type == "pana_switch"

    @property
    def is_dimmer(self) -> bool:
        """Return True if the device is a dimmer"""
        return self.device_type == "dimmer"

    @property
    def is_dimmable(self) -> bool:
        """Return  True if the device is dimmable."""
        return False

    @property
    def friendly_name(self) -> str:
        """Return a friendly description of the device"""
        return f"Swidget {self.device_type} w/{self.insert_type} insert"

    @property  # type: ignore
    def is_on(self) -> bool:
        """Return whether device is on."""
        dimmer_state = self.assemblies['host'].components["0"].functions['toggle']["state"]
        if dimmer_state == "on":
            return True
        return False

    async def turn_on(self):
        """Turn the device on."""
        await self.send_command(
            assembly="host", component="0", function="toggle", command={"state": "on"}
        )
    async def turn_off(self):
        """Turn the device off."""
        await self.send_command(
            assembly="host", component="0", function="toggle", command={"state": "off"}
        )

    async def turn_on_usb_insert(self):
        """Turn the USB insert on."""
        await self.send_command(
            assembly="insert", component="usb", function="toggle", command={"state": "on"}
        )

    async def turn_off_usb_insert(self):
        """Turn the USB insert off."""
        await self.send_command(
            assembly="insert", component="usb", function="toggle", command={"state": "off"}
        )

    @property  # type: ignore
    def usb_is_on(self) -> bool:
        """Return whether USB is on."""
        usb_state = self.assemblies['insert'].components["usb"].functions['toggle']["state"]
        if usb_state == "on":
            return True
        return False

    def __repr__(self):
        if self._last_update is None:
            return f"<{self.device_type} at {self.ip_address} - update() needed>"
        return f"<{self.device_type} model {self.model} at {self.ip_address}>"

    def __del__(self):
        if self.use_websockets:
            self.stop()


class SwidgetAssembly:
    def __init__(self, summary: dict):
        self.type = summary["type"]
        self.components = {
            c["id"]: SwidgetComponent(c["functions"]) for c in summary["components"]
        }
        self.id = summary.get("id")
        self.error = summary.get("error")


class SwidgetComponent:
    def __init__(self, functions):
        self.functions = {f: None for f in functions}


import time

import requests


def provision_wifi(ssid, network_password, token_name, secret_key):
    payload = {"ssid": ssid, "password": network_password, "secretKey": secret_key}
    headers = {token_name: secret_key}
    url = "https://10.123.45.1/network"
    initial_provision = requests.post(url, data=payload, verify=False).json()
    print(f"Response from setting {initial_provision}")

    time.sleep(5)
    url = "https://10.123.45.1/network"
    verify_connection = requests.get(url, headers=headers, verify=False).json()
    print(f"Verification request data: {verify_connection}")


import logging

from swidget.swidgetdevice import (
    DeviceType,
    SwidgetDevice
)
from swidget.exceptions import SwidgetException



log = logging.getLogger(__name__)

class SwidgetDimmer(SwidgetDevice):

    def __init__(self, host,  token_name: str, secret_key: str, ssl: bool, use_websockets: bool) -> None:
        super().__init__(host=host, token_name=token_name, secret_key=secret_key, ssl=ssl, use_websockets=use_websockets)
        self._device_type = "dimmer"

    @property  # type: ignore
    def brightness(self) -> int:
        """Return current brightness on dimmers.

        Will return a range between 0 - 100.
        """
        if not self.is_dimmable:
            raise SwidgetException("Device is not dimmable.")
        try:
            return self.assemblies['host'].components["0"].functions["level"]["now"]
        except KeyError:
            return self.assemblies['host'].components["0"].functions["level"]["default"]

    async def set_brightness(self, brightness):
        """Set the brightness of the device."""
        await self.send_command(
            assembly="host", component="0", function="level", command={"now": brightness}
        )

    async def set_default_brightness(self, brightness):
        await self.send_command(
            assembly="host", component="0", function="level", command={"default": brightness}
        )

    @property  # type: ignore
    def is_dimmable(self) -> bool:
        """Whether the switch supports brightness changes."""
        return True


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['swidget']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.1',
 'anyio',
 'asyncclick>=8',
 'importlib-metadata',
 'pydantic>=1,<2',
 'ssdp==1.1.1']

extras_require = \
{'docs': ['sphinx>=4,<5',
          'm2r>=0,<1',
          'mistune<2.0.0',
          'sphinx_rtd_theme>=0,<1',
          'sphinxcontrib-programoutput>=0,<1']}

entry_points = \
{'console_scripts': ['swidget = swidget.cli:cli']}

setup_kwargs = {
    'name': 'python-swidget',
    'version': '0.0.20',
    'description': 'Python API for Swidget smart devices',
    'long_description': '# python-swidget\nA library to manage Swidget smart devices\n',
    'author': 'Swidget',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/swidget/python-swidget',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
